#IF not %def(%PB_DATE_BAS)
%PB_DATE_BAS= 1

'==============================================================================
'
'  pb_date.bas
'
'  Modified by Don Dickinson - original file from powerbasic
'
'  Date routines for PB/DLL
'  Copyright (c) 1996 by PowerBASIC, Inc.  All Rights Reserved.
'
'
'  NOTE:  All routines expect dates and times to be in U.S. format.
'         For international support, all you need to do is modify the
'         SplitDate(), SplitTime(), MakeDate$(), MakeTime12$()
'         routines to support your specified format.  All other routines
'         call these routines to get the component parts.
'
'         All dates used must be between 01-01-1900 and 06-05-2079
'
'         If you pass a null (zero-length) string to a given routine,
'         the current date/time is used.
'
'
'  MODIFICATION HISTORY:
'    4-9-2001 Don Dickinson:
'           Added support for FormatDt function
'           Added conditional compilate constant %DATETIME_BAS
'
'------------------------------------------------------------------------------
'------------------------------------------------------------------------------
'
'  CDOW           -  Return the name for the Day of the Week of a date
'  DOW            -  Return the day of the week for a given date
'  DOY            -  Return the day of the year for a given date
'  DateMath       -  Add/Subtract days to a given date
'  DateToDays     -  Convert a date into the number of days since 1900
'  Days           -  Return the number of days between two dates
'  DaysToDate     -  Convert the number of days since 1900 into a date
'  Elapsed        -  Return the number of seconds between two time strings
'  isLeap           -  Check a year to see if it's a leap year
'  MakeDate       -  Create a date string from month, day, and year
'  MakeTime       -  Create a time string from hour, minutes, and seconds
'  MakeTime12     -  Create a 12 hour time string from hour and minutes
'  MonthName      -  Return the name of a given month
'  SecondsToTime  -  Convert number of seconds since midnight into a time
'  SplitDate      -  Split a date into its component parts
'  SplitTime      -  Split a time into its component parts
'  TimeToSeconds  -  Convert a time string into seconds since midnight
'  ValidDate      -  Check a date to see if it's valid
'  FormatDt       -  Format's a date similarly to VB
'
'------------------------------------------------------------------------------


'===========================================================================
' MonthName - Return the name for a specified month.
'
' Month = month to return name for
'
Sub MonthName(ByVal Month As Long, RetVal As String)

  If (Month < 1) Or (Month > 12) Then
    Exit Sub
  End If

  RetVal = Read$(Month)

  Data January, February, March, April, May, June
  Data July, August, September, October, November, December

End Sub

'===========================================================================
' MonthNameAbbrev - Return the name for a specified month.
'
' Month = month to return name for
'
Sub MonthNameAbbrev(ByVal Month As Long, RetVal As String)

  If (Month < 1) Or (Month > 12) Then
    Exit Sub
  End If

  RetVal = Read$(Month)

  Data Jan, Feb, Mar, Apr, May, June
  Data July, Aug, Sept, Oct, Nov, Dec

End Sub


'===========================================================================
' DayName - Return the name for a specified day of the week.
'
' Day = day to return name for
'
Sub DayName(ByVal Day As Long, RetVal As String)

  If (Day < 1) Or (Day > 7) Then
    Exit Sub
  End If

  RetVal = Read$(Day)

  Data Sunday, Monday, Tuesday, Wednesday, Thursday, Friday, Saturday

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  getMonthName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function getMonthName(ByVal iMonth as Long) as String
   select case iMonth
      Case 1
         function = "January"
      Case 2
         function = "February"
      case 3
         function = "March"
      case 4
         function = "April"
      case 5
         function = "May"
      case 6
         function = "June"
      case 7
         function = "July"
      case 8
         function = "August"
      case 9
         function = "September"
      case 10
         function = "October"
      case 11
         function = "November"
      case 12
         function = "December"
   end select

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  getShortMonthName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function getShortMonthName(ByVal iMonth as Long) as String
   function = left$(getMonthName(iMonth), 3)
end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  getDayName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function getDayName( ByVal iDay as Long ) as String

   Select Case iDay
      Case 1
         function = "Sunday"
      Case 2
         function = "Monday"
      case 3
         function = "Tuesday"
      case 4
         Function = "Wednesday"
      case 5
         function = "Thursday"
      case 6
         function = "Friday"
      case else
         function = "Saturday"
   End select
end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  getShortDayName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function getShortDayName( ByVal iDay as Long ) as String

   function = left$(getDayName(iDay), 3)

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  getReallyShortDayName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function getReallyShortDayName( ByVal iDay as Long ) as String

   Select Case iDay
      Case 1
         function = "Su"
      Case 2
         function = "M"
      Case 3
         function = "Tu"
      Case 4
         function = "W"
      Case 5
         function = "Th"
      Case 6
         function = "F"
      Case 7
         function = "Sa"
   End Select

end function

'===========================================================================
' DayNameAbbrev - Return the name for a specified day of the week.
'
' Day = day to return name for
'
Sub DayNameAbbrev(ByVal Day As Long, RetVal As String)

  If (Day < 1) Or (Day > 7) Then
    Exit Sub
  End If

  RetVal = Read$(Day)

  Data Sun, Mon, Tues, Wed, Thur, Fri, Sat

End Sub


'===========================================================================
' SplitDate - Split a date passed in a string to its Month, Day, and Year
'             components.
'
'             Note:  All other date routines call this routine to get the
'                    component parts.
'
' InDate = Date string in U.S. format (MM-DD-YY or MM-DD-YYYY)
' Month  = Returns the month
' Day    = Returns the day of the month
' Year   = Returns the year
'
Sub SplitDate(sInDate As String, iMonth As Long, iDay As Long, iYear As Long)

   If Len(sInDate) = 0 Then
      sInDate = Date$
   End If

   Replace "/" with "-" in sInDate
   iMonth = int(val(parse$(sInDate, "-", 1)))
   iDay = int(val(parse$(sInDate, "-", 2)))
   iYear = int(val(parse$(sInDate, "-", 3)))

   If iYear < 80 Then        ' assume we've rolled into the next century
      iYear = iYear + 2000
   ElseIf iYear < 100 Then   ' between 80 and 99, use this century
      iYear = iYear + 1900
   End If

End Sub


'===========================================================================
' SplitTime - Split a time passed in a string to its Hour, Minute, and
'             Second components.
'
'             Note:  All other time routines call this routine to get the
'                    component parts.
'
' TempTime = Time string in U.S. format (HH:MM:SS, HH:MM, HH:MMx)
' Hour     = Returns the hour (in military time)
' Minute   = Returns the minute
' Second   = Returns the second
'
Sub SplitTime(InTime As String, _
              Hours As Long, _
              Minutes As Long, _
              Seconds As Long)

  If Len(InTime) = 0 Then
    InTime$ = Time$
  End If

  Hours   = Val(InTime)
  Minutes = Val(Mid$(InTime,4,2))
  If Len(InTime) > 6 Then
    Seconds = Val(Mid$(InTime,7))
  Else
    Seconds = 0
  End If

  If Instr(Left$(InTime,2), Any "pP") Then
    If Hours < 12 Then
      Hours = Hours + 12
    End If
  ElseIf Instr(Left$(InTime, 2), Any "aA") Then
    If Hours = 12 Then
      Hours  = 0
    End If
  End If

End Sub


'===========================================================================
' MakeDate - Combine the component parts of a date into a displayable
'            string in U.S. format.
'
'             Note:  All other date routines call this routine to combine
'                    the component parts.
'
Sub MakeDate(ByVal Month As Long, _
             ByVal Day As Long, _
             ByVal Year As Long, _
             lpStr As String)

  Dim TmpStr As String

  If Year < 80 Then        'assume we've rolled into the next century
    Year = Year + 2000
  ElseIf Year < 100 Then   'between 1980 and 1999
    Year = Year + 1900
  End If

  TmpStr = Right$(Str$(Month), 2) + "-" + _
           Right$(Str$(Day), 2) + "-" + _
           Right$(Str$(Year), 4)

  Replace " " With "0" In TmpStr
  lpStr = TmpStr

End Sub


'===========================================================================
' MakeTime12 - Combine the component parts of a time into a displayable
'              string in 12 hour U.S. format.
'
Sub MakeTime12(ByVal Hour As Long, _
               ByVal Minute As Long, _
               lpStr As String)

  Dim Pm     As Long
  Dim H      As String
  Dim M      As String
  Dim TmpStr As String

  If Hour > 12 Then
    Hour = Hour -12
    Pm   = -1
  End If

  H = Right$(Str$(Hour),2)
  M = Right$(Str$(Minute),2)

  If Pm Then
    M = M + "p"
  Else
    M = M + "a"
  End If

  TmpStr = H + ":" + M
  Replace " " With "0" In TmpStr
  lpStr = TmpStr

End Sub


'===========================================================================
' MakeTime - Combine the component parts of a time into a displayable
'            string in 24 hour U.S. format.
'
'             Note:  All other time routines call this routine to combine
'                    the component parts.
'
Sub MakeTime(ByVal Hour As Long, _
             ByVal Minute As Long, _
             ByVal Second As Long, _
             lpStr As String)

  Dim H      As String
  Dim M      As String
  Dim S      As String
  Dim TmpStr As String

  lpStr = Format$(Hour, "00") + ":" + _
          Format$(Minute, "00") + ":" + _
          Format$(Second, "00")
  exit sub

'  H = Right$(Str$(Hour),2)
'  M = Right$(Str$(Minute),2)
'  S = Right$(Str$(Second),2)

'  TmpStr = H + ":" + M + ":" + S
'  Replace " " With "0" In TmpStr

'  lpStr = TmpStr

End Sub


'===========================================================================
' isLeap - Return a one (1) if specified year is a leap year.
'
' Year = year to check
'
Function isLeap(ByVal Year As Long)  As Long

  If (Year Mod 400) = 0 Then
    Function = 1
  ElseIf (Year Mod 100) = 0 Then
    Function = 0
  ElseIf (Year Mod 4) = 0 Then
    Function = 1
  End If

End Function


'===========================================================================
' DaysInMonth - Return the number of days in a given month.  It does not
'               account for leap years.
'
' Month = month to return number of days for
'
Function DaysInMonth(ByVal Month As Long)  As Long

  If (Month > 0) And (Month < 13) Then
    Function = Val( Read$(Month) )
  End If

  Data 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31

End Function


'===========================================================================
' DateToDays - Convert the specified date into the number of days since
'              01-01-1900.  Supports dates up to 06-05-2079.
'
' InDate = Date to convert
'
Function DateToDays(InDate As String)  As Long

  Dim Temp   As Long
  Dim i      As Long
  Dim Month  As Long
  Dim Day    As Long
  Dim Year   As Long

  SplitDate InDate, Month, Day, Year

  If Year < 80 Then
    Year = Year + 2000
  ElseIf Year < 100 Then
    Year = Year + 1900
  ElseIf Year < 1900 Then
    Exit Function
  End If

  If Year > 1900 Then
    For i = Year - 1 To 1900 Step -1
      Temp = Temp + 365 + isLeap(i)
    Next i
  End If

  Temp = Temp + Val( Read$(Month) )

  If (Month > 2) Then
    Temp = Temp + isLeap(Year)
  End If

  Temp = Temp + Day

  Function = Temp

  Data 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 366

End Function


'===========================================================================
' DaysToDate - Convert the number of days since 01-01-1900 to a date string.
'
' NumDays = number of days since 1900
'
'  Altered by don to return a value instead of returning an altered parameter
'
Function DaysToDate(ByVal NumDays As Long) as String

  Dim Month        As Long
  Dim Year         As Long
  Dim DaysInMnth   As Long
  Dim DaysInYear   As Long
  Dim TmpStr       As String
  Dim RetVal As String


  Year       = 1900
  DaysInYear = 365 + isLeap(Year)

  Do Until (NumDays < DaysInYear)
    NumDays = NumDays - (365 + isLeap(Year))
    Incr Year
    DaysInYear = 365 + isLeap(Year)
  Loop

  For Month = 1 To 12
    DaysInMnth = DaysInMonth(Month)
    If NumDays <= DaysInMnth Then
      Exit For
    End If
    NumDays = NumDays - DaysInMnth
  Next

  If NumDays = 0 Then
    Month = 12
    NumDays = 31
    Decr Year
  End If

  MakeDate Month, NumDays, Year, RetVal
  Function = RetVal

End Function


'===========================================================================
' DateMath - Add the specified number of days the specified date.
'            To subtract days, use a negative value.
'
' InDate       = starting date
' NumberOfDays = number of days to add
'
Sub DateMath(InDate As String, ByVal NumberOfDays As Long, _
             RetVal As String)

  RetVal = DaysToDate(DateToDays(InDate) + NumberOfDays)

End Sub


'===========================================================================
' Days - Return the number of days between two dates.  If the first date
'        comes after the second date, a negative value is returned.
'
' Date1 = starting date
' Date2 = ending date
'
Function Days(Date1 As String, Date2 As String)  As Long

  Function = DateToDays(Date1) - DateToDays(Date2)

End Function


'===========================================================================
' ValidDate - Returns true (-1) if the specified date is valid, or false (0)
'             if the date is invalid.
'
' InDate = date to check
'
Function ValidDate(InDate As String)  As Long

  Dim Month     As Long
  Dim Day       As Long
  Dim Year      As Long
  Dim TotalDays As Long

  If Len(InDate) = 0 Then
    Exit Function
  End If

  SplitDate InDate, Month, Day, Year

  If (Month < 1) Or (Month > 12) Then
    Exit Function
  End If

  TotalDays = DaysInMonth(Month)

  If Month = 2 Then
    TotalDays = TotalDays + isLeap(Year)
  End If

  If (Day < 1) Or (Day > TotalDays) Then
    Exit Function
  End If

  If Year < 1900 Then
    Exit Function
  End If

  Function = -1  ' a valid date

End Function


'===========================================================================
' TimeToSeconds - Returns the number of seconds since midnight for a given
'                 time string.
'
' InTime = Time to convert into seconds
'
Function TimeToSeconds(InTime As String)  As Long

  Dim Hour   As Long
  Dim Minute As Long
  Dim Second As Long

  SplitTime InTime, Hour, Minute, Second

  Function = (Hour * 3600) + (Minute * 60) + Second

End Function


'===========================================================================
' SecondsToTime$ - Return a time string for a given number of seconds since
'                  midnight.
'
' Secs = seconds to convert into a time string
'  Altered by don to return a string instead of altering a parameter
'
Function SecondsToTime(ByVal Secs As Long) as String

   Dim Hour   As Long
   Dim Minute As Long
   Dim Second As Long
   Dim RetVal As String

   Hour   = Secs \ 3600
   Minute = (Secs Mod 3600) \ 60
   Second = Secs - (hour * 3600) - (Minute * 60)

   MakeTime Hour, Minute, Second, RetVal
   Function = RetVal

End Function


'===========================================================================
' Elapsed - Return the number of seconds elapsed between two time strings.
'           If the second time is earlier than the first, a midnight rollover
'           is assumed.
'
' Time1 = start time
' Time2 = end time
'
Function Elapsed(Time1 As String, Time2 As String)  As Long

  Dim Temp1 As Long
  Dim Temp2 As Long

  Temp1 = TimeToSeconds(Time1)
  Temp2 = TimeToSeconds(Time2)

  If Temp2 < Temp1 Then  'account for midnight rollover
    Temp2 = Temp2 + 86400&
  End If

  Function = Temp2 - Temp1

End Function


'===========================================================================
' DOW - Return the Day of the Week as an Long.
'
' InDate = date to convert into the day of the week
'
Function DOW(InDate As String)  As Long

  Function = 1 + DateToDays(InDate) Mod 7

End Function


'===========================================================================
' DOY - Return the Day of the Year
'
' InDate = date to convert into day of the year
'
Function DOY(InDate As String)  As Long

  Dim Month  As Long
  Dim Day    As Long
  Dim Year   As Long
  Dim Temp   As Long

  SplitDate InDate, Month, Day, Year

  Temp = Val( Read$(Month) )

  If (Month > 2) Then Temp = Temp + isLeap(Year)

  Function = Temp

  Data 0, 31, 59, 090, 120, 151, 181, 212, 243, 273, 304, 334, 366

End Function


'===========================================================================
' CDOW - Return the name for the Day of the Week for the specified date.
'
' InDate = Date to return day of week name for
'
Sub CDOW(InDate As String, RetVal As String)

  DayName DOW(InDate), RetVal

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  FormatDt
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function FormatDt(Byval incoming as String, ByVal dateFormat as String) as String

   Dim m as Long
   Dim d as Long
   Dim y as Long
   Dim mon as String
   Dim monAbbrev as String
   Dim dw as String
   Dim dowAbbrev as String
   Dim sReturn as String

   If ValidDate(incoming) = %False then
      Function = incoming
      exit function
   end if

   SplitDate incoming, m, d, y
   MonthName m, mon
   MonthNameAbbrev m, monAbbrev
   DayName DOW(incoming), dw
   DayNameAbbrev DOW(incoming), dowAbbrev

   sReturn = lcase$(dateFormat)

   '- It is necessary to do this in two steps to
   '  avoid the "m" in monday to be replaced
   '  with the month, or the d in monday to
   '  be replaced with the day, etc.
   '
   Replace "yyyy" with chr$(1) in sReturn
   Replace "yy" with chr$(2) in sReturn
   Replace "mmmm" with chr$(3) in sReturn
   Replace "mmm" with chr$(4) in sReturn
   Replace "mm" with chr$(5) in sReturn
   Replace "m" with chr$(6) in sReturn
   Replace "dddd" with chr$(7) in sReturn
   Replace "ddd" with chr$(8) in sReturn
   Replace "dd" with chr$(9) in sReturn
   Replace "d" with chr$(10) in sReturn

   Replace chr$(1) with format$(y, "0000") in sReturn
   Replace chr$(2) with right$(format$(y, "0000"), 2) in sReturn
   Replace chr$(3) with mon in sReturn
   Replace chr$(4) with monAbbrev in sReturn
   Replace chr$(5) with format$(m, "00") in sReturn
   Replace chr$(6) with format$(m) in sReturn
   Replace chr$(7) with dw in sReturn
   Replace chr$(8) with dowAbbrev in sReturn
   Replace chr$(9) with format$(d, "00") in sReturn
   Replace chr$(10) with format$(d) in sReturn

   Function = sReturn

End Function
#ENDIF