'
'  pb_email.bas
'
'  DESCRIPTION
'     This module will compile into PBDLL6 or PBCC2 programs
'     and add smtp email capabilities. Also provides .ini file
'     handling, ras, and some file operations. When compiled
'     with pbdll, it has input box and progress bar functionality
'     as well.
'
'  COPYRIGHT AND AUTHOR
'     Copyright 1999-2002 Don Dickinson All rights reserved.
'     www.basicguru.com/dickinson
'     ddickinson@usinternet.com
'
'  LICENSE
'     This code is provided in good faith by the author, Don Dickinson.
'     You are welcome to use it for whatever purpose serves your needs,
'     By doing so, you are agreeing to hold the author harmless from
'     all effects and side effects of its use or mis-use. Use at your own risk.
'
'  NOTES
'     The only function you should use from PBCC is pbSendEmailEx
'     This sends email without a dialog. If you need the ability
'     to use the dialogs, you need to compile this with PBDLL
'     as a dll and then call it from your pbcc app via a dll call.
'
'  DEPENDS ON
'     win32api.inc         from powerbasic
'     commctrl.inc         from powerbasic (pbcc doesn't require this)
'     comdlg32.inc         from powerbasic (pbcc doesn't require this)
'     pb_fonts.bas         from don dickinson (pbcc doesn't require this)
'     pb_input.bas         from don dickinson (pbcc doesn't require this)
'     pb_prog.bas          from don dickinson (pbcc doesn't require this)
'     pb_file.bas          from don dickinson
'     pb_ras.bas           from don dickinson
'     pb_ini.bas           from don dickinson
'
'  HISTORY
'     >> 1-20-2002 don dickinson
'        Added pbcc support by placing conditional compilation
'        (#if %def(%pb_dll32)) around all pbdll specific calls
'        Basically, I just removed the progress bar when compiling
'        with pbcc.
'
'
'
#if not %def(%PB_EMAIL_BAS)

%PB_EMAIL_BAS = 1

'- Generic PB 32-bit functions
#include "win32api.inc"    'windows api declares from powerbasic

'- This can compile into a pbcc program by removing the
'  dialog status stuff
'
#if %def(%pb_dll32)
   #include "commctrl.inc"    'common control declares from powerbasic
   #include "comdlg32.inc"    'common dialog declares from powerbasic
   #Include "pb_fonts.bas"    'font creation routines from Don Dickinson
   #Include "pb_input.bas"    'input box wrapper from Don Dickinson
   #Include "pb_prog.bas"     'progress bar wrapper from Don Dickinson
#endif
#Include "pb_file.bas"     'file manipulation from Don Dickinson
#Include "pb_mime.bas"     'mime routines from Dave Navarro, enhanced by Don Dickinson
#Include "pb_ras.bas"      'ras dialing wrapper from Don Dickinson
#Include "pb_ini.bas"      'ini file handing from Don Dickinson
#include "pb_date.bas"     'mutilation of datetime.bas from PB, by Don Dickinson

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Constants and Structures
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

'- Structures
Type SMTP_OPTIONS_TYPE
   smtpServer As String * 300
   smtpPort As Long
   smtpReply As String * 300
   smtpUseRas As Long
   smtpRasEntry As String * 300
   smtpRasUser as Asciiz * 100
   smtpRasPassword as Asciiz * 100
End Type

'- Version
%PBEMAIL_BUILD_NUMBER          =  5

'- error classes
%PBEMAIL_100   = 100
%PBEMAIL_200   = 200
%PBEMAIL_300   = 300
%PBEMAIL_400   = 400
%PBEMAIL_500   = 500

'- Debug, beta and evaluation constants
'%DEBUG_ATTACHMENT_NAMES = 1
'%DEBUG_ENCODING         = 1
'%DEBUG_LOG              = 1
'%PBEMAIL_EVALUATION_VERSION = 1
'%PBEMAIL_BETA_VERSION = 1

'- Return constants for pbSendEMail
%PBEMAIL_SUCCESS               =  0
%PBEMAIL_INVALID_OPTIONS       = -1
%PBEMAIL_USER_CANCEL           = -2
%PBEMAIL_INVALID_RAS_ENTRY     = -3
%PBEMAIL_RAS_DIAL_FAILED       = -4

'- ini file sections, mime border
$PBEMAIL_DEFAULT_INI_SECTION = "pbemail"
$PBEMAIL_MIME_BOUNDARY = "===================6B9767D121AE========="

'- Constants for pbEmailLastEmailField
%PBEMAIL_FIELD_ADDR          = 0
%PBEMAIL_FIELD_SUBJECT       = 1
%PBEMAIL_FIELD_NOTE          = 2
%PBEMAIL_FIELD_FILES         = 3


'- Constants needed for dialog generation
%PBEMAIL_LABEL_WIDTH   = 35
%PBEMAIL_LABEL_LEFT    = 5
%PBEMAIL_TEXT_WIDTH    = 100
%PBEMAIL_TEXT_WIDTH2   = 265
%PBEMAIL_TEXT_LEFT     = 45
%PBEMAIL_BTN_WIDTH     = 40
%PBEMAIL_BTN_HEIGHT    = 14
%PBEMAIL_DLG_WIDTH     = %PBEMAIL_TEXT_LEFT + %PBEMAIL_TEXT_WIDTH + 15

%txt_pbemail_Server         = 200
%txt_pbemail_User           = 201
%txt_pbemail_Port            = 202
%txt_pbemail_Addr           = 203
%txt_pbemail_Subject        = 204
%txt_pbemail_Note           = 205
%txt_pbemail_Reply          = 206
%txt_pbemail_UserID         = 207
%txt_pbemail_Password       = 208
%btn_pbemail_Ok             = 300
%btn_pbemail_Cancel         = 301
%btn_pbemail_AddFile        = 302
%btn_pbemail_DelFile        = 303
%btn_pbemail_Setup          = 304
%cbo_pbemail_Ras            = 400
%opt_pbemail_NoDial         = 500
%opt_pbemail_Dial           = 501
%lst_pbemail_Files          = 700
%lab_pbemail_Status         = 800
%hidden_pbemail_iniFile     = 900
%hidden_pbemail_iniSection  = 901

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Globals
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

'- data exchange with dialogs
Global g_EMailAddr As String
Global g_EMailSubject As String
Global g_EMailNote As String
Global g_EMailFiles As String
Global g_EMailFileNames As String
Global g_EMailParent As Long

Global g_EmailLogin as String
Global g_TimeZoneOffset as String

'- dialog handles
Global dlgOptions As Long
Global dlgEMail As Long

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  Function Prototypes
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Declare Function pbSendEMail Alias "pbSendEMail" _
      ( ByVal hParent As Long, zAddr As Asciiz, _
        zIniFile As Asciiz, zIniSection As Asciiz, _
        zSubject As Asciiz, zNote As Asciiz, _
        zFile As Asciiz, zFileNames As Asciiz, _
        ByVal iShowDialog As Long ) As Long

Declare Function pbSendEMailEx Alias "pbSendEMailEx" _
      ( ByVal hParent As Long, zServer As Asciiz, _
        zReply As Asciiz, zRasEntry As Asciiz, _
        zRasUser As Asciiz, zRasPassword As Asciiz, _
        zAddr As Asciiz, zSubject As Asciiz, _
        zNote As Asciiz, zFile As Asciiz, _
        zFileNames As Asciiz, ByVal iBlind as Long, _
        ByVal iProgress as Long ) As Long

Declare Function pbEmailLastEMailField Alias "pbEmailLastEMailField" _
      ( ByVal iWhichField As Long ) As String

Declare Function pbEmailLastEMailFieldZ Alias "pbEmailLastEMailFieldZ" _
      ( ByVal iWhichField As Long ) As Dword

Declare Function pbEmailOptions Alias "pbEmailOptions" _
    ( ByVal hParent As Long, zIniFile As Asciiz, _
      zIniSection As Asciiz ) As Long

Declare Function pbEmailOptionsValid Alias "pbEmailOptionsValid" _
      (zIniFile As Asciiz, zIniSection As Asciiz ) As Long

Declare Function pbEmailBuild Alias "pbEmailBuild" As Long

Declare Function smtpValidOptions _
      ( rSMTP As SMTP_OPTIONS_TYPE ) As Long

Declare Function smtpGetOptions _
      ( ByVal iniFileName As String, ByVal iniSect As String, _
        rSMTP As SMTP_OPTIONS_TYPE ) As Long

Declare Function smtpSetOptions _
      ( ByVal iniFileName As String, ByVal iniSect As String, _
        rSMTP As SMTP_OPTIONS_TYPE) As Long

Declare Sub smtpPrepNoteAndFile _
      ( ByVal hProg As Long, sMsg As String, sFiles As String, _
        sFileNames As String, sLines() As String)

Declare Function SmtpSendMail _
      ( ByVal hProg As Long, _
        ByVal SmtpHost As String, ByVal EmailFrom As String, _
        ByVal sEmailTo As String, ByVal Subject As String, _
        Message() As String, ByVal iBlind as Long ) As Long


#if %def(%pb_dll32)
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  BEGIN: OPTIONS DIALOG BOX
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  dlgOptions_btnOk_Click
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CallBack Function dlgOptions_btnOk_Click

   Dim iPos as Long
   Dim iDial As Long
   Dim sText As String
   Dim iniFile as String
   Dim iniSection as String
   Dim saveSMTP As SMTP_OPTIONS_TYPE

   '- The filename is in the caption
   '  of the dialog right after the
   '  first instance of a :
   '
   Control Get Text dlgOptions, %hidden_pbemail_iniFile to iniFile
   Control Get Text dlgOptions, %hidden_pbemail_iniSection to iniSection

   Control Get Text dlgOptions, %txt_pbemail_Server To sText
   If Trim$(sText) = "" Then
      MsgBox "You must supply a server name!", 64, "SMTP Setup"
      Exit Function
   Else
      saveSMTP.smtpServer = Trim$(sText)
   End If

   Control Get Text dlgOptions, %txt_pbemail_Reply To sText
   If Trim$(sText) = "" Then
      MsgBox "You must supply a reply address!", 64, "SMTP Setup"
      Exit Function
   Else
      saveSMTP.smtpReply = Trim$(sText)
   End If

   Control Get Text dlgOptions, %txt_pbemail_Port To sText
   If Trim$(sText) = "" Then
      MsgBox "You must supply a TCP Port!", 64, "SMTP Setup"
      Exit Function
   ElseIf Int(Val(sText)) > 65000 Or Int(Val(sText)) < 1 Then
      MsgBox "Invalid port setting. Setting of 25 is normal. The value must be between 1 and 65000", 64, _
             "SMTP Setup"
      Exit Function
   Else
      saveSMTP.smtpPort = Int(Val(sText))
   End If

   Control Get Check dlgOptions, %opt_pbemail_Dial To iDial
   If iDial Then
      saveSMTP.smtpUseRas = %True
      ComboBox Get Text dlgOptions, %cbo_pbemail_Ras To sText
      If Trim$(sText) = "" Then
         MsgBox "You must select a RAS entry to dial. " + _
                "Please consult your system administrator if you don't know how to setup your SMTP options.", _
                64, "SMTP Setup"
         Exit Function
      End If
      saveSMTP.smtpRasEntry = Trim$(sText)
      Control Get Text dlgOptions, %txt_pbemail_UserID to sText
      if Trim$(sText) = "" then
         MsgBox "You must type a User ID for the Ras Entry. " + _
                "The password can be left blank, but the User ID " + _
                "must be specified!", 64, "SMTP Setup"
         exit function
      end if
      saveSMTP.smtpRasUser = Trim$(sText)

      Control Get Text dlgOptions, %txt_pbemail_Password to sText
      saveSMTP.smtpRasPassword = Trim$(sText)
   Else
      saveSMTP.smtpUseRas = %False
      saveSMTP.smtpRasEntry = ""
      saveSMTP.smtpRasUser = ""
      saveSMTP.smtpRasPassword = ""
   End If

   smtpSetOptions iniFile, iniSection, saveSMTP

   Dialog End dlgOptions, 1

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  dlgOptions_btnCancel_Click
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CallBack Function dlgOptions_btnCancel_Click

  Dialog End dlgOptions, 0

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  dlgOptions_optDial_Click
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CallBack Function dlgOptions_optDial_Click

   Dim iCheck As Long

   Control Get Check dlgOptions, %opt_pbemail_Dial To iCheck
   If iCheck Then
      Control Enable dlgOptions, %cbo_pbemail_Ras
      Control Enable dlgOptions, %txt_pbemail_UserID
      Control Enable dlgOptions, %txt_pbemail_Password
   Else
      Control Disable dlgOptions, %cbo_pbemail_Ras
      Control Disable dlgOptions, %txt_pbemail_UserID
      Control Disable dlgOptions, %txt_pbemail_Password
   End If

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  dlgOptions_Create
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function dlgOptions_Create(ByVal hParent As Long, ByVal iniFile as String, _
                           ByVal iniSection as String) As Long

   Dim i As Long
   Dim iNumRas As Long
   Dim sRasEntries(1 To 1) As String
   Dim rSMTP As SMTP_OPTIONS_TYPE

   '- Get the RAS entries on this system
   iNumRas = rsAvailableEntries()
   If iNumRas > 0 Then
      ReDim sRasEntries(1 To iNumRas)  As String
      For i = 1 To iNumRas
         sRasEntries(i) = rsGetEntryName(i)
      Next i
   Else
      ReDim sRasEntries(0 To 0) As String
   End If

   '- Get email options
   smtpGetOptions iniFile, iniSection, rSMTP

   Dialog New hParent, "SMTP EMail Options", , , %PBEMAIL_DLG_WIDTH, 190, _
         %WS_POPUPWINDOW or %WS_CAPTION, To dlgOptions

   '- LABELS
   Control Add Label, dlgOptions, %hidden_pbemail_iniFile, iniFile, _
         0, 300, 10, 10, 0
   Control Add Label, dlgOptions, %hidden_pbemail_iniSection, iniSection, _
         0, 300, 10, 10, 0

   Control Add Label, dlgOptions, -1, "Server", _
         %PBEMAIL_LABEL_LEFT, 10, %PBEMAIL_LABEL_WIDTH, 15, %SS_RIGHT
   Control Add Label, dlgOptions, -1, "Port", _
         %PBEMAIL_LABEL_LEFT, 30, %PBEMAIL_LABEL_WIDTH, 15, %SS_RIGHT
   Control Add Label, dlgOptions, -1, "Reply Address", _
         %PBEMAIL_LABEL_LEFT, 50, %PBEMAIL_LABEL_WIDTH, 15, %SS_RIGHT
   Control Add Label, dlgOptions, -1, "UserID", _
         %PBEMAIL_LABEL_LEFT, 120, %PBEMAIL_LABEL_WIDTH, 15, %SS_RIGHT
   Control Add Label, dlgOptions, -1, "Password", _
         %PBEMAIL_LABEL_LEFT, 140, %PBEMAIL_LABEL_WIDTH, 15, %SS_RIGHT

   '- TEXT BOXES
   Control Add TextBox, dlgOptions, %txt_pbemail_Server, Trim$(rSMTP.smtpServer), _
         %PBEMAIL_TEXT_LEFT,  10, %PBEMAIL_TEXT_WIDTH, _
         15, %ES_AUTOHSCROLL Or %WS_BORDER Or %WS_TABSTOP,%WS_EX_CLIENTEDGE
   Control Add TextBox, dlgOptions, %txt_pbemail_Port, Format$(rSMTP.smtpPort), _
         %PBEMAIL_TEXT_LEFT,  30, %PBEMAIL_TEXT_WIDTH, 15, _
         %ES_AUTOHSCROLL Or %WS_BORDER Or %WS_TABSTOP,%WS_EX_CLIENTEDGE
   Control Add TextBox, dlgOptions, %txt_pbemail_Reply, Trim$(rSMTP.smtpReply), _
         %PBEMAIL_TEXT_LEFT,  50, %PBEMAIL_TEXT_WIDTH,  15, _
         %ES_AUTOHSCROLL Or %WS_BORDER Or %WS_TABSTOP,%WS_EX_CLIENTEDGE


   '- RADIO BUTTONS
   Control Add Option, dlgOptions, %opt_pbemail_NoDial, "I am connected directly to the internet.", _
         %PBEMAIL_TEXT_LEFT - 40, 75, %PBEMAIL_TEXT_WIDTH + 40, 12,, Call dlgOptions_optDial_Click
   Control Add Option, dlgOptions, %opt_pbemail_Dial, "Dial the internet using ....", _
         %PBEMAIL_TEXT_LEFT - 40, 85, %PBEMAIL_TEXT_WIDTH + 40, 12,, Call dlgOptions_optDial_Click

   '- COMBOBOX
   Control Add ComboBox, dlgOptions, %cbo_pbemail_Ras, sRasEntries(), _
         %PBEMAIL_TEXT_LEFT - 30, 100, %PBEMAIL_TEXT_WIDTH + 30, 90, _
         %WS_BORDER Or %WS_TABSTOP Or %CBS_DROPDOWNLIST, %WS_EX_CLIENTEDGE

   '- USER AND PASSWORD
   Control Add TextBox, dlgOptions, %txt_pbemail_UserID, Trim$(rSMTP.smtpRasUser), _
         %PBEMAIL_TEXT_LEFT,  120, %PBEMAIL_TEXT_WIDTH, _
         15, %ES_AUTOHSCROLL Or %WS_BORDER Or %WS_TABSTOP,%WS_EX_CLIENTEDGE
   Control Add TextBox, dlgOptions, %txt_pbemail_Password, Trim$(rSMTP.smtpRasPassword), _
         %PBEMAIL_TEXT_LEFT,  140, %PBEMAIL_TEXT_WIDTH, _
         15, %ES_AUTOHSCROLL Or %WS_BORDER Or %WS_TABSTOP,%WS_EX_CLIENTEDGE


   '- BUTTONS
   Control Add Button, dlgOptions, %IDCANCEL, "&Cancel", _
         %PBEMAIL_DLG_WIDTH - %PBEMAIL_BTN_WIDTH - 10, 165, %PBEMAIL_BTN_WIDTH, %PBEMAIL_BTN_HEIGHT, _
         0 Call dlgOptions_btnCancel_Click
   Control Add Button, dlgOptions, %IDOK,  "&Save", _
         %PBEMAIL_DLG_WIDTH - 2 * %PBEMAIL_BTN_WIDTH - 15, 165, %PBEMAIL_BTN_WIDTH, %PBEMAIL_BTN_HEIGHT, _
         %BS_DEFPUSHBUTTON Or %WS_TABSTOP Call dlgOptions_btnOk_Click

   '- Find the ras entry in the list if they're using
   '  RAS to dial the internet.
   '
   If rSMTP.smtpUseRas Then
      Control Set Check dlgOptions, %opt_pbemail_Dial, 1
      Control Enable dlgOptions, %cbo_pbemail_Ras
      Control Enable dlgOptions, %txt_pbemail_UserID
      Control Enable dlgOptions, %txt_pbemail_Password

      If iNumRas > 0 Then
         For i = 1 To iNumRas
            If Trim$(UCase$(sRasEntries(i))) = Trim$(UCase$(rSMTP.smtpRasEntry)) Then
               ComboBox Select dlgOptions, %cbo_pbemail_Ras, i
               Exit For
            End If
         Next i
      End If

   Else
      Control Set Check dlgOptions, %opt_pbemail_NoDial, 1
      Control Disable dlgOptions, %cbo_pbemail_Ras
      Control Disable dlgOptions, %txt_pbemail_UserID
      Control Disable dlgOptions, %txt_pbemail_Password
   End If

End Function
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  END: OPTIONS DIALOG BOX
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  BEGIN: EMAIL DIALOG BOX
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  dlgEMail_btnSend_Click
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CallBack Function dlgEMail_btnSend_Click

   Dim e As Long
   Dim i As Long
   Dim iFiles As Long
   Dim iChecked As Long
   Dim hFiles As Long
   Dim zText As Asciiz * 400
   Dim sText As String

   Control Get Text dlgEMail, %txt_pbemail_Addr To sText
   If Trim$(sText) = "" Then
      MsgBox "You must supply at lease one recipient's email address. " + _
             "If you have more than one recipient, separate their " + _
             "addresses with semi-colons or commas.", 64, "SMTP EMail"
      Exit Function
   Else
      g_EMailAddr = Trim$(sText)
   End If

   Control Get Text dlgEMail, %txt_pbemail_Subject To sText
   If Trim$(sText) = "" Then
      MsgBox "You must supply a subject line for your email.", _
             64, "SMTP EMail"
      Exit Function
   Else
      g_EMailSubject = Trim$(sText)
   End If

   Control Get Text dlgEMail, %txt_pbemail_Note To sText
   g_EMailNote = sText

   '- The files need to be moved from the list box
   '  into a delimited string.
   '
   g_EMailFiles = ""
   Control Handle dlgEMail, %lst_pbemail_Files To hFiles
   iFiles = SendMessage(hFiles, %LB_GETCOUNT, 0, 0)
   For i = 0 To iFiles - 1
      SendMessage hFiles, %LB_GETTEXT, i, VarPtr(zText)
      If Trim$(zText) <> "" Then
         g_EMailFiles = g_EMailFiles + ";" + Trim$(zText)
      End If
   Next i
   If Len(g_EMailFiles) > 1 Then g_EMailFiles = Mid$(g_EMailFiles, 2)

   Dialog End dlgEMail, 1

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  dlgEMail_btnCancel_Click
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CallBack Function dlgEMail_btnCancel_Click

   Dialog End dlgEMail, 0

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  dlgEMail_btnAddFile_Click
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CallBack Function dlgEMail_btnAddFile_Click

   Dim sFile As String

   If OpenFileDialog(g_EMailParent, "Attach File", sFile, "", "*.*", "", _
         %OFN_FILEMUSTEXIST Or %OFN_HIDEREADONLY) Then
      ListBox Add dlgEMail, %lst_pbemail_Files, sFile
   End If

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  dlgEMail_btnRemoveFile_Click
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CallBack Function dlgEMail_btnRemoveFile_Click

   Dim i As Long
   Dim hList As Long
   Dim iSelected As Long
   Dim sFile As String

   '- Which one is selected
   Control Handle dlgEMail, %lst_pbemail_Files To hList
   i = SendMessage(hList, %LB_GETCURSEL, 0, 0)
   If i < 0 Then
      Exit Function
   End If

   ListBox Delete dlgEMail, %lst_pbemail_Files, i + 1

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  dlgEMail_btnSetup_Click
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CallBack Function dlgEMail_btnSetup_Click

   Dim iniFile as String
   Dim iniSection as String
   Dim zIniFile as Asciiz * 300
   Dim zIniSection as Asciiz * 300

   Control Get Text dlgEMail, %hidden_pbemail_iniFile to iniFile
   Control Get Text dlgEMail, %hidden_pbemail_iniSection to iniSection
   zIniFile = Trim$(iniFile)
   zIniSection = Trim$(iniSection)
   pbEmailOptions dlgEMail, zIniFile, zIniSection

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  dlgEMail_Create
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function dlgEMail_Create( ByVal hParent As Long, iniFile as string, _
                          iniSection as String, sAddr As String, _
                          sSubject As String, sNote As String,  _
                          sFile As String ) Export As Long

   Dim i As Long
   Dim iFiles As Long
   Dim iCount As Long
   Dim sFiles() As String

   ReDim sFiles(0 To 0) As String

   g_EMailParent = hParent

   iFiles = ParseCount(sFile, Any ";,")
   For i = 1 To iFiles
      If Trim$(Parse$(sFile, Any ";,", i)) <> "" Then
         iCount = iCount + 1
         If iCount = 1 Then
            ReDim sFiles(1 To 1) As String
         Else
            ReDim Preserve sFiles(1 To iCount) As String
         End If
         sFiles(iCount) = Trim$(Parse$(sFile, Any ";,", i))
      End If
   Next i

   Dialog New hParent, "Send Internet EMail", , , %PBEMAIL_DLG_WIDTH * 2, 230, _
              %WS_POPUPWINDOW or %WS_CAPTION,  To dlgEMail

   '- LABELS
   Control Add Label, dlgOptions, %hidden_pbemail_iniFile, iniFile, _
         0, 300, 10, 10, 0
   Control Add Label, dlgOptions, %hidden_pbemail_iniSection, iniSection, _
         0, 300, 10, 10, 0

   Control Add Label, dlgEMail, -1, "EMail Addresses", _
         %PBEMAIL_LABEL_LEFT, 10, %PBEMAIL_LABEL_WIDTH, 15, %SS_RIGHT
   Control Add Label, dlgEMail, -1, "Subject", _
         %PBEMAIL_LABEL_LEFT, 70, %PBEMAIL_LABEL_WIDTH, 15, %SS_RIGHT
   Control Add Label, dlgEMail, -1, "Note", _
         %PBEMAIL_LABEL_LEFT, 90, %PBEMAIL_LABEL_WIDTH, 15, %SS_RIGHT
   Control Add Label, dlgEMail, -1, "Files", _
         %PBEMAIL_LABEL_LEFT, 155, %PBEMAIL_LABEL_WIDTH, 15, %SS_RIGHT

   '- TEXT BOXES
   Control Add TextBox, dlgEMail, %txt_pbemail_Addr, sAddr, _
         %PBEMAIL_TEXT_LEFT,  10, %PBEMAIL_TEXT_WIDTH2, 58, _
         %ES_AUTOVSCROLL Or %ES_MULTILINE Or %ES_WANTRETURN Or _
         %WS_TABSTOP, %WS_EX_CLIENTEDGE

   Control Add TextBox, dlgEMail, %txt_pbemail_Subject, sSubject, _
         %PBEMAIL_TEXT_LEFT,  70, %PBEMAIL_TEXT_WIDTH2,  15, _
         %ES_AUTOHSCROLL Or %WS_BORDER Or %WS_TABSTOP,%WS_EX_CLIENTEDGE
   Control Add TextBox, dlgEMail, %txt_pbemail_Note, sNote, _
         %PBEMAIL_TEXT_LEFT, 90, %PBEMAIL_TEXT_WIDTH2, 60, _
         %ES_AUTOVSCROLL Or %ES_MULTILINE Or %ES_WANTRETURN Or _
         %WS_TABSTOP,%WS_EX_CLIENTEDGE

   '- LIST BOXES
   Control Add ListBox, dlgEMail, %lst_pbemail_Files, sFiles(), _
                        %PBEMAIL_TEXT_LEFT, 155, _
                        %PBEMAIL_TEXT_WIDTH + 30, 50, 0, _
                        %WS_EX_CLIENTEDGE

   '- BUTTONS
   Control Add Button, dlgEMail, %btn_pbemail_AddFile, "&Add File", _
      %PBEMAIL_TEXT_LEFT + %PBEMAIL_TEXT_WIDTH + 35, 155, _
      %PBEMAIL_BTN_WIDTH + 10, %PBEMAIL_BTN_HEIGHT,, _
      Call dlgEMail_btnAddFile_Click
   Control Add Button, dlgEMail, %btn_pbemail_DelFile, "&Remove File", _
      %PBEMAIL_TEXT_LEFT + %PBEMAIL_TEXT_WIDTH + 35, _
      155 + %PBEMAIL_BTN_HEIGHT + 5, _
      %PBEMAIL_BTN_WIDTH + 10, %PBEMAIL_BTN_HEIGHT,, _
      Call dlgEMail_btnRemoveFile_Click

   Control Add Button, dlgEMail, %IDCANCEL, "&Cancel", _
      (%PBEMAIL_DLG_WIDTH * 2) - %PBEMAIL_BTN_WIDTH - 10, 210, _
      %PBEMAIL_BTN_WIDTH, %PBEMAIL_BTN_HEIGHT, 0 _
      Call dlgEMail_btnCancel_Click
   Control Add Button, dlgEMail, %IDOK,  "&Send", _
      (%PBEMAIL_DLG_WIDTH * 2) - 2 * %PBEMAIL_BTN_WIDTH - 15, 210, _
      %PBEMAIL_BTN_WIDTH, %PBEMAIL_BTN_HEIGHT, %BS_DEFPUSHBUTTON Or %WS_TABSTOP _
      Call dlgEMail_btnSend_Click

   Control Add Button, dlgEMail, %btn_pbemail_Setup, "&Setup ...", _
      %PBEMAIL_TEXT_LEFT, 210, %PBEMAIL_BTN_WIDTH, %PBEMAIL_BTN_HEIGHT, 0 _
      Call dlgEMail_btnSetup_Click


End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  END: EMAIL DIALOG BOX
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbEmailOptions                                                    EXPORTED
'
'     Displays a dialog that allows the user to setup
'     their SMTP options.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function pbEmailOptions Alias "pbEmailOptions" _
            ( ByVal hParent As Long, zIniFile As Asciiz, zIniSection As Asciiz ) _
            Export As Long

   Dim iResult As Long

   dlgOptions_Create hParent, Trim$(zIniFile), Trim$(zIniSection)
   Dialog Show Modal dlgOptions To iResult
   If iResult Then
      Function = %True
   Else
      Function = %False
   End If

End Function


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbEmailLastEMailField                                                EXPORTED
'
'     Returns the text typed in the dialogs of the
'     last pbSendMail Call As a String
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function pbEmailLastEMailField Alias "pbEmailLastEMailField" _
      ( ByVal iWhichField As Long ) Export As String

   Dim sReturn As String

   Select Case iWhichField
      Case %PBEMAIL_FIELD_ADDR
         sReturn = g_EMailAddr
      Case %PBEMAIL_FIELD_SUBJECT
         sReturn = g_EMailSubject
      Case %PBEMAIL_FIELD_NOTE
         sReturn = g_EMailNote
      Case %PBEMAIL_FIELD_FILES
         sReturn = g_EMailFiles
      Case Else
         sReturn = ""
   End Select

   Function = sReturn

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbzEmailLastEMailField                                               x
'
'     Returns the text typed in the dialogs of the last pbSendMail call
'     as an asciiz pointer
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function pbzEmailLastEMailField Alias "pbzEmailLastEMailField" _
      ( ByVal iWhichField As Long ) Export As Dword

   Dim sReturn As String

   sReturn = pbEmailLastEMailField(iWhichField) + Chr$(0)
   Function = StrPtr(sReturn)

End Function

#endif

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  webDate
'
'  Formats a date according to web standard
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function webDate(ByVal sDate as String, ByVal sTime as String) as String

   Dim iDOW as Long
   Dim iYr as Long
   Dim iMon as Long
   Dim iDay as Long

   iDOW = DOW(sDate)
   SplitDate sDate, iMon, iDay, iYr
   function = getShortDayName(iDOW) + ", " + _
              format$(iDay) + " " + _
              getShortMonthName(iMon) + " " + _
              format$(iYr) + " " + _
              sTime + " " + _
              g_TimeZoneOffset

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbEmailSetDateOffset
'  This is the date time zone qualifier. It would be (for example)
'  -05:00 for CST in the US. It's the hours relative to GMT
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub pbEMailSetDateOffset Alias "pbEmailSEtDateOffset" _
      ( ByVal sOffset as String )

   g_TimeZoneOffset = sOffset

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbEmailOptionsValid                                                 EXPORTED
'
'  Displays a dialog box giving the user an opportunity to change
'  their smtp setup options.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function pbEmailOptionsValid Alias "pbEmailOptionsValid" _
      (zIniFile As Asciiz, zIniSection As Asciiz ) Export As Long

   Dim rSMTP As SMTP_OPTIONS_TYPE

   If smtpGetOptions(zIniFile, zIniSection, rSMTP) = %False Then
      Function = %False
   Else
      Function = smtpValidOptions(rSMTP)
   End If

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbEmailBuild                                                        EXPORTED
'
'  Returns the build number (version) of pbemail.dll
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function pbEmailBuild Alias "pbEmailBuild" Export As Long
   Function = %PBEMAIL_BUILD_NUMBER
End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbEmailSetLogin
'
'  This tells the client what to add use the client's name in
'  HELO smtp login command. Most servers won't care what this
'  is, but some do. By default, the system will use the host
'  name as returned by PB's HOST command. If that returns empty,
'  "localhost" will be used.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub pbEmailSetLogin Alias "pbEmailSetLogin" ( zLoginAs as Asciiz ) Export

   g_EmailLogin = zLoginAs

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbSendEMailEx                                                        EXPORTED
'
'  Sends email without a dialog box and doesn't use the system
'  defaults for reply address, server, and rasentry. It also
'  supresses the recipient list if the iBlind option is set.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function pbSendEMailEx Alias "pbSendEMailEx" _
      ( ByVal hParent As Long, zServer As Asciiz, zReply As Asciiz, _
        zRasEntry As Asciiz, zRasUser As Asciiz, zRasPassword As Asciiz, _
        zAddr As Asciiz, zSubject As Asciiz, zNote As Asciiz, _
        zFile As Asciiz, zFileNames As Asciiz, ByVal iBlind as Long, _
        ByVal iProgress as Long ) Export As Long

   Dim e As Long
   Dim iResult As Long
   Dim iFound As Long
   Dim iDialedHere As Long
   Dim iActive As Long
   Dim iEntry As Long
   Dim hRas As Long
   Dim hProgress As Long
   Dim iLog As Long
   Dim zDialer As Asciiz * 300
   Dim sError As String
   Dim sLines() As String


   iLog = 0
   sError = ""
   #If %def(%DEBUG_LOG)
      iLog = FreeFile
      Open "pbemail.log" For Append As #iLog
   #endif

   #If %Def(%PBEMAIL_EVALUATION_VERSION)
   MsgBox "This program is calling an Evaluation version of Don Dickinson's " + $cr + _
          "pbEmail library." + $cr + "http://dickinson.basicguru.com" + $cr + _
          "ddickinson@usinternet.com", 64, "pbEmail Evaluation"
   #EndIf
   #If %Def(%PBEMAIL_BETA_VERSION)
   MsgBox "This program is calling a BETA version of Don Dickinson's " + $cr + _
          "pbEMail library." + $cr + "http://dickinson.basicguru.com" + $cr + _
          "ddickinson@usinternet.com", 64, "pbEMail BETA"
   #EndIf
   g_EMailFileNames = zFileNames

   g_EMailAddr = zAddr
   g_EMailSubject = zSubject
   g_EMailNote = zNote
   g_EMailFiles = zFile
   iResult = %True

   ReDim sLines(0 To 0) As String

   '- Send the email
   If iResult = %False Then
      sError = "FAIL: USER CANCEL iResult = False before any calls. This shouldn't happen."
      Function = %PBEMAIL_USER_CANCEL
   Else

#if %def(%pb_dll32)
      if iProgress then
         hProgress = pbProgressInit(0, "Sending EMail")
         pbProgressSet hProgress, 0, "Validating E-Mail Options"
      else
         hProgress = 0
      end if
#endif

      '- Dial RAS if necessary
      If Trim$(zRasEntry) <> "" Then
#if %def(%pb_dll32)
         if iProgress then
            pbProgressSet hProgress, 0, "Dialing Connection " + zRasEntry + " ..."
         end if
#endif
         zDialer = Trim$(zRasEntry)
         iEntry = rsValidEntry(zDialer)
         If iEntry = 0 Then
            sError = "FAIL: Invalid Ras Entry: " + Trim$(zRasEntry)
            Function = %PBEMAIL_INVALID_RAS_ENTRY
            GoTo SendBlind_Abort_Send
         End If

         iActive = rsActiveConnections()
         If iActive < 1 Then
            iDialedHere = %True
            hRas = rsDialEntryEx(iEntry, zRasUser, zRasPassword)
            If hRas = 0 Then
               sError = "FAIL: Ras Dial Failed"
               Function = %PBEMAIL_RAS_DIAL_FAILED
               GoTo SendBlind_Abort_Send
            End If
         End If
      End If

#if %def(%pb_dll32)
      if iProgress then
         If IsWindow(hProgress) Then
            If pbProgressIsCancelled(hProgress) Then
               sError = "Progress bar cancel"
               Function = %PBEMAIL_USER_CANCEL
               GoTo SendBlind_Abort_Send
            End If
         End If
         pbProgressSet hProgress, 20, "Preparing Attachments"
      end if
#endif

      smtpPrepNoteAndFile hProgress, g_EMailNote, g_EMailFiles, g_EMailFileNames, sLines()

#if %def(%pb_dll32)
      if iProgress then
         pbProgressSet hProgress, 60, "Sending"
      end if
#endif

      e = SmtpSendMail(hProgress, Trim$(zServer), Trim$(zReply), g_EMailAddr, _
            g_EMailSubject, sLines(), %True )
      If e = 0 Then
         sError = "SENT OK!"
         Function = %PBEMAIL_SUCCESS
      Else
         sError = "SMTP Error: " + Format$(e)
         Function = e
      End If
   End If

SendBlind_Abort_Send:

   #If %def(%DEBUG_LOG)
      Print #iLog, sError
   #endif

#if %def(%pb_dll32)
   '- Kill the dialog
   if iProgress then pbProgressShutdown hProgress
#endif

   '- Hangup RAS if we dialed it here.
   If (Trim$(zRasEntry) <> "") And (iDialedHere <> 0) Then
      rsHangupConnection hRas
   End If

End Function

'- pbSendEMail is not available to pbcc unless called
'  in a compiled dll version of pb_email
'
#if %def(%pb_dll32)


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbSendEMail                                                       EXPORTED
'
'  Sends the email message as specified. If SMTP isn't setup, this will
'  call pbSetMTPOptions and send the message after being setup.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function pbSendEMail Alias "pbSendEMail" _
      ( ByVal hParent As Long, zAddr As Asciiz, _
        zIniFile As Asciiz, zIniSection As Asciiz, _
        zSubject As Asciiz, zNote As Asciiz, _
        zFile As Asciiz, zFileNames As Asciiz, _
        ByVal iShowDialog As Long ) Export As Long

   Dim e As Long
   Dim iResult As Long
   Dim iFound As Long
   Dim iDialedHere As Long
   Dim iActive As Long
   Dim iEntry As Long
   Dim hRas As Long
   Dim hProgress As Long
   Dim zDialer As Asciiz * 300
   Dim sRasPassword As String
   Dim sLines() As String
   Dim inSMTP As SMTP_OPTIONS_TYPE

   #If %Def(%PBEMAIL_EVALUATION_VERSION)
   MsgBox "This program is calling an Evaluation version of Don Dickinson's " + $cr + _
          "pbEmail library." + $cr + "http://dickinson.www.basicguru.com" + $cr + _
          "ddickinson@usinternet.com", 64, "pbEMail Evaluation"
   #EndIf
   #If %Def(%PBEMAIL_BETA_VERSION)
   MsgBox "This program is calling a BETA version of Don Dickinson's " + $cr + _
          "pbEmail library." + $cr + "http://dickinson.basicguru.com" + $cr + _
          "ddickinson@usinternet.com", 64, "pbEmail BETA"
   #EndIf

   '- Make sure that we have valid setup options
   smtpGetOptions Trim$(zIniFile), Trim$(zIniSection$), inSMTP
   If smtpValidOptions(inSMTP) = %False Then
      If pbEmailOptions(hParent, zIniFile, zIniSection) = %False Then
         Function = %PBEMAIL_INVALID_OPTIONS
         Exit Function
      End If
   End If

   g_EMailFileNames = zFileNames

   '- Prompt for email information
   If iShowDialog Then
      dlgEMail_Create hParent, Trim$(zIniFile), Trim$(zIniSection), _
            Trim$(zAddr), Trim$(zSubject), Trim$(zNote), Trim$(zFile)
      Dialog Show Modal dlgEMail To iResult
      If Trim$(UCase$(zFile)) <> Trim$(UCase$(g_EMailFiles)) Then
         g_EMailFileNames = ""
      End If
   Else
      g_EMailAddr = zAddr
      g_EMailSubject = zSubject
      g_EMailNote = zNote
      g_EMailFiles = zFile
      iResult = %True
   End If

   ReDim sLines(0 To 0) As String

   '- Send the email
   If iResult = %False Then
      Function = %PBEMAIL_USER_CANCEL
   Else

      hProgress = pbProgressInit(0, "Sending EMail")
      pbProgressSet hProgress, 0, "Retrieving E-Mail Options"

      smtpGetOptions Trim$(zIniFile), Trim$(zIniSection), inSMTP

      '- Dial RAS if necessary
      If inSMTP.smtpUseRas Then
         zDialer = Trim$(inSMTP.smtpRasEntry)
         iEntry = rsValidEntry(zDialer)
         If iEntry = 0 Then
            Function = %PBEMAIL_INVALID_RAS_ENTRY
            GoTo Send_Abort_Send
         End If

         iActive = rsActiveConnections()
         If iActive < 1 Then
            If Trim$(inSMTP.smtpRasPassword) = "" Then
               If pbInputbox( hParent, "Password Needed", _
                  "Please enter the password for your Ras Account: " + _
                  Trim$(inSMTP.smtpRasUser),  "", sRasPassword, %False) = %False Then
                  Function = %PBEMAIL_USER_CANCEL
                  GoTo Send_Abort_Send
               Else
                  inSMTP.smtpRasPassword = Trim$(sRasPassword)
               End If
            End If
'
            pbProgressSet hProgress, 0, "Dialing " + zDialer + "..."
            iDialedHere = %True
            hRas = rsDialEntryEx(iEntry, inSMTP.smtpRasUser, inSMTP.smtpRasPassword)
            If hRas = 0 Then
               Function = %PBEMAIL_RAS_DIAL_FAILED
               GoTo Send_Abort_Send
            End If
         End If
      End If

      If pbProgressIsCancelled(hProgress) Then
         Function = %PBEMAIL_USER_CANCEL
         GoTo Send_Abort_Send
      End If

      smtpPrepNoteAndFile hProgress, g_EMailNote, g_EMailFiles, g_EMailFileNames, sLines()

      e = SmtpSendMail(    hProgress, Trim$(inSMTP.smtpServer), Trim$(inSMTP.smtpReply), _
                        g_EMailAddr, g_EMailSubject, sLines(), %False )
      If e = 0 Then
         Function = %PBEMAIL_SUCCESS
      Else
         Function = e
      End If
   End If

Send_Abort_Send:

   '- Kill the dialog
   pbProgressShutdown hProgress

   '- Hangup RAS if we dialed it here.
   If (inSMTP.smtpUseRas <> 0) And (iDialedHere <> 0) Then
      rsHangupConnection hRas
   End If

End Function
#endif

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  smtpValidOptions
'
'  Lets us know if the smtp setup information is at least
'  good enough to save (not blank)
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function smtpValidOptions( rSMTP As SMTP_OPTIONS_TYPE ) As Long

   If (Trim$(rSMTP.smtpServer) = "")  Or _
      (rSMTP.smtpPort < 1) Or _
      (Trim$(rSMTP.smtpReply) = "")Then
      Function = %False
   ElseIf (rSMTP.smtpUseRas <> %False) And _
          (Trim$(rSMTP.smtpRasUser) = "") Then
      Function = %False
   Else
      Function = %True
   End If

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  smtpGetOptions
'
'     Retrieves the options structure from an ini file
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function smtpGetOptions( ByVal iniFileName As String, ByVal iniSect As String, _
                         rSMTP As SMTP_OPTIONS_TYPE ) As Long

   Dim sResult As String
   Dim iniFile As String
   Dim iniSection As String

   iniSection = Trim$(iniSect)
   If iniSection = "" Then
      iniSection = $PBEMAIL_DEFAULT_INI_SECTION
   End If
   iniFile = Trim$(iniFileName)
   If iniFile = "" Then
      iniFile = "win.ini"
   End If

   '- Set Defaults
   rSMTP.smtpServer = ""
   rSMTP.smtpPort = 25

   rSMTP.smtpServer = GetIni(iniFile, iniSection, "smtpServer", "")
   rSMTP.smtpPort = Int(Val(GetIni(iniFile, iniSection, "smtpPort", "25")))
   If rSMTP.smtpPort < 1 Then rSMTP.smtpPort = 25

   rSMTP.smtpReply = GetIni(iniFile, iniSection, "smtpReply", "")
   sResult = Left$(Trim$(GetIni(iniFile, iniSection, "smtpUseRas", "N")) + "N", 1)
   If (sResult = "Y") Or (sResult = "1") Then
      rSMTP.smtpUseRas = %True
      rSMTP.smtpRasEntry = GetIni(iniFile, iniSection, "smtpRasEntry", "")
      rSMTP.smtpRasUser = GetIni(iniFile, iniSection, "smtpRasUser", "")
      rSMTP.smtpRasPassword = GetIni(iniFile, iniSection, "smtpRasPassword", "")
   Else
      rSMTP.smtpUseRas = %False
      rSMTP.smtpRasEntry = ""
      rSMTP.smtpRasUser = ""
      rSMTP.smtpRasPassword = ""
   End If

   Function = smtpValidOptions(rSMTP)

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  smtpSetOptions
'
'  Write the options structure to an ini file
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function smtpSetOptions( ByVal iniFileName As String, ByVal iniSect As String, _
                         rSMTP As SMTP_OPTIONS_TYPE) As Long

   Dim sResult As String
   Dim iniSection As String
   Dim iniFile As String

   iniSection = Trim$(iniSect)
   If iniSection = "" Then
      iniSection = $PBEMAIL_DEFAULT_INI_SECTION
   End If
   iniFile = Trim$(iniFileName)
   If iniFile = "" Then
      iniFile = "win.ini"
   End If

   WriteIni iniFile, iniSection, "smtpServer", Trim$(rSMTP.smtpServer)
   WriteIni iniFile, iniSection, "smtpPort", Format$(rSMTP.smtpPort)
   WriteIni iniFile, iniSection, "smtpReply", Trim$(rSMTP.smtpReply)
   If rSMTP.smtpUseRas Then
      WriteIni iniFile, iniSection, "smtpUseRas", "Y"
      WriteIni iniFile, iniSection, "smtpRasEntry", Trim$(rSMTP.smtpRasEntry)
      WriteIni iniFile, iniSection, "smtpRasUser", Trim$(rSMTP.smtpRasUser)
      WriteIni iniFile, iniSection, "smtpRasPassword", Trim$(rSMTP.smtpRasPassword)
   Else
      WriteIni iniFile, iniSection, "smtpUseRas", "N"
      WriteIni iniFile, iniSection, "smtpRasEntry", ""
      WriteIni iniFile, iniSection, "smtpRasUser", ""
      WriteIni iniFile, iniSection, "smtpRasPassword", ""
   End If


   '- This really could be a sub as it always returns %True
   Function = %True

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  smtpPrepNoteAndFile
'
'  Takes the note string and breaks into an array of strings. It also
'  mime encodes any files requested and appends them to the array.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub smtpPrepNoteAndFile( ByVal hProg As Long, sMsg As String, sFiles As String, _
                         sFileNames As String, sLines() As String)

   Dim i As Long
   Dim iFF As Long
   Dim iParse As Long
   Dim iLines As Long
   Dim iFiles As Long
   Dim iFileLines As Long
   Dim bignote As String
   Dim zIn As Asciiz * 300
   Dim zTemp As Asciiz * 300
   Dim zFN As Asciiz * 300
   Dim sF() As String
   Dim sFN() As String
   Dim sFileLines() As String
   Dim zBoundary As Asciiz * 300
   Dim sTemp As String

   '- Any crlf pairs or plain lf's will be
   '  replaced with cr's for easy parsing
   '
   bigNote = Trim$(sMsg)
   Replace Chr$(13,10) With Chr$(13) In bignote
   Replace Chr$(10) With Chr$(13) In bignote

   '- Break the Note into lines
   iLines = ParseCount(bigNote, $cr) + 5
   ReDim sLines(0 To iLines + 2) As String

   sLines(1) = "--" + $PBEMAIL_MIME_BOUNDARY

   if lcase$(left$(bigNote, 6))  = "<html>" then
      sLines(2) = "Content-Type: text/html; charset=" + Chr$(34) + "us-ascii" + Chr$(34)
   else
      sLines(2) = "Content-Type: text/plain; charset=" + Chr$(34) + "us-ascii" + Chr$(34)
   end if

   sLines(3) = "Content-Transfer-Encoding: 7bit"
   sLines(4) = ""
   For i =5 To iLines
      sLines(i) = Parse$(bigNote, $cr, i - 4)
   Next i

   #If %Def(%DEBUG_ENCODING)
   MsgBox "Note Parsed"
   #EndIf

   '- Get a list of files
   iFiles = 0
   iParse = ParseCount(sFiles, Any ";,")
   ReDim sF(0 To 0) As String
   ReDim sFN(0 To 0) As String
   For i = 1 To iParse
      sTemp = Trim$(Parse$(sFiles, Any ";,", i))
      If sTemp <> "" Then
         If Dir$(sTemp) <> "" Then
            iFiles = iFiles + 1
            ReDim Preserve sF(0 To iFiles) As String
            ReDim Preserve sFN(0 To iFiles) As String
            sF(i) = sTemp
            sFN(i) = Trim$(Parse$(sFileNames, Any ";,", i))
            #If %Def(%DEBUG_ATTACHMENT_NAMES)
            MsgBox "i =" + Str$(i) + $cr + _
                   "sF(i) = " + sF(i) + $cr + _
                   "sFN(i) = " + sFN(i) + $cr + _
                   "sFileNames = " + sFileNames + $cr + _
                   "sFiles = " + sFiles
            #EndIf

         End If
      End If
   Next i

   #If %Def(%DEBUG_ENCODING)
   MsgBox "File Names Parsed: " + Format$(iParse) + ":" + Format$(iFiles) + _
         Chr$(13) + sFiles
   #EndIf

   '- The MIME boundary - if there's no
   '  files to include, we're done - so exit.
   iLines = iLines + 1
   If iFiles = 0 Then
      sLines(iLines) = "--" + $PBEMAIL_MIME_BOUNDARY + "--"
      '=======
      Exit Sub
      '=======
   Else
      sLines(iLines) = "--" + $PBEMAIL_MIME_BOUNDARY
   End If

   '- Turn each file into a mime encoded file
   '  and read it back in.
   '
   On Error Resume Next
   For i = 1 To iFiles

      '- The file name must not be blank and
      '  it must exist in order to be encoded.
      '
      If sF(i) <> "" Then
         If Dir$(sF(i)) <> "" Then

            If i = iFiles Then
               zBoundary = "--" + $PBEMAIL_MIME_BOUNDARY + "--"
            Else
               zBoundary = "--" + $PBEMAIL_MIME_BOUNDARY
            End If

            '- Create the mime file
            zTemp = GetTempFile("pbe", ".mim")
            zIn = sF(i)
            zFN = sFN(i)
            #If %Def(%DEBUG_ENCODING)
            MsgBox "About to Mime encode file: " + zTemp
            #EndIf

            If pbFileToMIME(zIn, zTemp, zFN, zBoundary, hProg) Then

               '- Append the mime file to the array
               On Error Resume Next
               iFF = FreeFile
               Open zTemp For Input As #iFF
               If Err = 0 Then
                  Do Until Eof(iFF)
                     If Err Then Exit Do
                     iLines = iLines + 1
                     ReDim Preserve sLines(0 To iLines) As String
                     Line Input #iFF, sLines(iLines)
                  Loop

                  Close #iFF
               End If

               '- Kill the temporary file
               Kill zTemp
               On Error GoTo 0
            Else
               #If %Def(%DEBUG_ATTACHMENT_NAMES)
               MsgBox "pbFileToMIME failed."
               #EndIf
            End If
            #If %Def(%DEBUG_ENCODING)
            MsgBox "Done Encodeing file: " + zTemp
            #EndIf

         End If
      End If
   Next i

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  sendLine
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub sendLine ( ByVal hTCP as Long, ByVal sToSend as String )

   tcp print #hTCP, sToSend
'   stdout "client: " + sToSend

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  buffer_status
'
'  returns the class of return parameter and extracts the buffer
'  data (fixes sBuffer)
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function buffer_status(ByVal sBuffer as String) as Long

   Dim iNum as Long

'   stdout "server: " + sBuffer

   iNum = int(val(left$(sBuffer, 3)))

   sBuffer = mid$(sBuffer, 5)
   if iNum < 200 then
      function = %PBEMAIL_100
   elseif iNum < 300 then
      function = %PBEMAIL_200
   elseif iNum < 400 then
      function = %PBEMAIL_300
   elseif iNum < 500 then
      function = %PBEMAIL_400
   else
      function = %PBEMAIL_500
   end if

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  smtpSendMail
'
'  Send a text email message to a single recipient.  Returns zero (0) if
'  successful.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function SmtpSendMail(  ByVal hProg As Long, _
                        ByVal SmtpHost As String, ByVal EmailFrom As String, _
                        ByVal sEmailTo As String, ByVal Subject As String, _
                        Message() As String, ByVal iBlind as Long ) As Long

   Dim hTCP       As Long
   Dim e          As Long
   Dim i          As Long
   Dim u          As Long
   Dim x          As Long
   Dim iTo        As Long
   Dim iRecip     As Long
   Dim Buffer     As String
   Dim localhost  As String
   Dim ToField    As String
   Dim sTO()      As String
   Dim sBCC()     As String
   Dim EMailTo    As String

   ReDim sBCC(0 To 0) As String
   ReDim sTo(0 To 0) As String

'   stdout "host: " + smtphost
'   stdout "emailfrom: " + EmailFrom
'   stdout "emailto: " + sEmailTo
'   stdout "subject: " + subject
'   stdout "blind: " + Format$(iBlind)

   '- Get the recipients
   EMailTo = sEmailTo
   Replace $crlf With ";" In EMailTo
   Replace $cr With ";" In EMailTo
   Replace $lf With ";" In EMailTo

   toField = EMailTo

   iRecip = ParseCount(EMailTo, Any ";,")

   ReDim sTo(0 To iRecip) As String
   For i = 1 To iRecip
      sTo(i) = Parse$(EMailTo, Any ";,", i)
   Next i
   iTo = iRecip

   On Error GoTo SmtpSendMail_Error

#if %def(%pbdll_32)
   if hProg then

      '- Get the local computer's name
      If pbProgressIsCancelled(hProg) Then
         e = %PBEMAIL_USER_CANCEL
         GoTo SmtpSendMail_Done
      End If
      pbProgressSet hProg, 0, "Connecting to " + smtpHost
   end if
#endif

   if trim$(g_EMailLogin) = "" then
      Host Name To localhost
      If Trim$(localhost) = "" Then
         localhost = "localhost"
      End If
   else
      localhost = g_EMailLogin
   end if

   '- Connect to the SMTP server
   hTCP = FreeFile
   Tcp Open "smtp" At SmtpHost As hTCP timeout 10000
   do
      tcp line input #hTCP, Buffer
      e = buffer_status(Buffer)
      if e <> %PBEMAIL_200 then
         Goto SmtpSendMail_Done
      end if
      if mid$(Buffer, 4, 1) <> "-" then exit do
   loop

   '- Meet & greet the SMTP host
   sendLine hTCP, "HELO " & localhost
   tcp line input #hTCP, Buffer
   e = buffer_status(Buffer)
   if e <> %PBEMAIL_200 then
      Goto SmtpSendMail_Done
   end if

   sendLine hTCP, "MAIL FROM:<" & EmailFrom & ">"
   tcp line input #hTCP, Buffer
   e = buffer_status(Buffer)
   if e <> %PBEMAIL_200 then
      Goto SmtpSendMail_Done
   end if

#if %def(%pb_dll32)
   if hProg then

      ' *** ERROR CHECK
      If pbProgressIsCancelled(hProg) Then
         e = %PBEMAIL_USER_CANCEL
         sendLine hTCP, "RSET"
         tcp line input #hTCP, Buffer
         sendLine hTCP, "QUIT"
         tcp line input #hTCP, Buffer
         GoTo SmtpSendMail_Done
      End If
      pbProgressSet hProg, 0, "Sending Recipient List ..."
   end if
#endif

   '- Send the recipients
   For i = 1 To iTo
      If Trim$(sTo(i)) <> "" Then
         sendLine hTCP, "RCPT TO:" + sTo(i) + "<" & sTo(i) & ">"
         tcp line input #hTCP, Buffer
         e = buffer_status(Buffer)
         if e <> %PBEMAIL_200 then
            Goto SmtpSendMail_Done
         end If
      end if
   Next i

#if %def(%pb_dll32)
   ' *** ERROR CHECK
   if hProg then
      If pbProgressIsCancelled(hProg) Then
         e = %PBEMAIL_USER_CANCEL
         sendLine hTCP, "RSET"
         tcp line input #hTCP, Buffer
         sendLine hTCP, "QUIT"
         tcp line input #hTCP, Buffer
         GoTo SmtpSendMail_Done
      End If
      pbProgressSet hProg, 0, "Sending EMail Header"
   end if
#endif

   sendLine hTCP, "DATA"
   tcp line input #hTCP, Buffer
   e = buffer_status(Buffer)
   if (e < %PBEMAIL_200) or (e > %PBEMAIL_300) then
      Goto SmtpSendMail_Done
   end If


   '- Message header
   sendLine hTCP, "Date: " + webDate(Date$, Time$)
   sendLine hTCP, "From: " + EmailFrom
   sendLine hTCP, "X-Mailer: pbEMail for powerbasic"
   if iBlind then
      sendLine hTCP, "To: <recipient.list.suppressed>"
   else
      sendLine hTCP, "To: " + toField
   end if

   sendLine hTCP, "Subject: " & Subject
   sendLine hTCP, "MIME-Version: 1.0"
   sendLine hTCP, "Content-Type: multipart/mixed; boundary=" + _
         Chr$(34) + $PBEMAIL_MIME_BOUNDARY + Chr$(34)
   sendLine hTCP, ""

   '- Message text
   u = UBound(Message(1))
   For x = 1 To u

#if %def(%pb_dll32)
      ' *** ERROR CHECK
      if hProg then
         If pbProgressIsCancelled(hProg) Then
            e = %PBEMAIL_USER_CANCEL
            GoTo SmtpSendMail_Done
         End If
         pbProgressSet hProg, (x / u) * 100, "Sending E-Mail Data. Lines " + _
               Format$(x) + " of " + Format$(u)
         If pbProgressIsCancelled(hProg) Then
            e = %PBEMAIL_USER_CANCEL
            GoTo SmtpSendMail_Done
         End If
      end if
#endif

      Buffer = Message(x)
      If Left$(Buffer, 1) = "." Then
         Buffer = "." + Buffer
      End If
      Tcp Print hTCP, Buffer
   Next x

   '- End of message
   tcp Print hTCP, "."
   tcp line input #hTCP, Buffer
   e = buffer_status(Buffer)
   if e <> %PBEMAIL_200 then
      goto SmtpSendMail_Done
   end if

   sendLine hTCP, "QUIT"
   tcp line input #hTCP, Buffer
   e = buffer_status(Buffer)
   if e <> %PBEMAIL_200 then
      goto SmtpSendMail_Done
   end if

   e = 0  'success!

SmtpSendMail_Done:
   Function = e
   Close hTCP
   Exit Function

SmtpSendMail_Error:
   e = Err
   Resume SmtpSendMail_Done

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbEmailDecodeError
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function pbEmailDecodeError Alias "pbEmailDecodeError" _
      ( ByVal iCode as Long ) export as String

   select case iCode
      case %PBEMAIL_SUCCESS            '0
         function = "Success. No errors encountered"
      case %PBEMAIL_INVALID_OPTIONS    ' -1
         function = "Fail. Invalide setup options found."
      case %PBEMAIL_USER_CANCEL
         function = "Fail. User cancel."
      case %PBEMAIL_INVALID_RAS_ENTRY
         function = "Fail. Invalid RAS entry."
      case %PBEMAIL_RAS_DIAL_FAILED
         function = "Fail. RAS dial was not successful."
      case else
         function = "SMTP Error: " + format$(abs(iCode))
   end select

end function

#endif
