'
'  pb_file.bas
'
'  PB-32bit misc file routines
'
'  Function FileDate(ByVal sFileName as String, ByVal sFormat as String as String
'  Function FileSize(ByVal sFileName as String) as Long
'  Function FileExists(sFile As String) As Long
'  Function ExtractPath (zFullFile As String) As String
'  Function ExtractFileName(zFillFile as String) as String
'  Function GetExeName() As String
'  Function GetModuleName(ByVal sModule as String) as String
'  Function GetTempFile(ByVal sSeed, ByVal sExt as String) As String
'  Function GetFileDateTime(sFileName as String) as String
'  Function GetFileSize(sFileName as String) as String
'  Function GetWindowsDir() as String
'  Function GetSystemDir() as String
'  Function GetTemporaryDir() as String
'  Sub DeleteFileSpec(ByVal sFileSpec As String)
'  Sub EnsureDirectoryExists(ByVal sFileName As String)
'  Function pbWriteFile(filename,filedata) as Long
'  Function ShellDocument(filename)
'
'  DEPENDS UPON
'     win32api.inc
'

#if not %def(%PB_FILE_BAS)
%PB_FILE_BAS = 1

#include "win32api.inc"

Global g_pb_file_LastError as String

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbFileLastError
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function pbFileLastError Alias "pbFileLastError" Export as String

   Function = g_pb_file_LastError

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  FileDate
'
'  Returns file date as a string formatted in the specified manner.
'  This is the date which the file was last written to.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function FileDate Alias "FileDate" _
      ( ByVal sFileName as String, ByVal sFormat as String) Export as String

   Dim hFind as Long
   Dim ampm as String
   Dim rData as WIN32_FIND_DATA
   Dim rTime as SYSTEMTIME

   hFind = FindFirstFile(sFileName + $nul, rData)
   if hFind = %INVALID_HANDLE_VALUE then
      Function = "n/a"
   else
      FileTimeToSystemTime rData.ftLastWriteTime, rTime

      sFormat = Ucase$(sFormat)
      if instr(sFormat, "A/P") > 0 then
         if rTime.wHour > 12 then
            rTime.wHour = rTime.wHour - 12
            ampm = "p"
         elseif rTime.wHour = 12 then
            ampm = "p"
         else
            ampm = "a"
         end if

      elseif instr(sFormat, "AM/PM") > 0 then
         if rTime.wHour > 12 then
            rTime.wHour = rTime.wHour - 12
            ampm = "pm"
         elseif rTime.wHour = 12 then
            ampm = "pm"
         else
            ampm = "am"
         end if
      else
         ampm = ""
      end if

      if len(ampm) > 0 then
         Replace "A/P" with ampm in sFormat
         Replace "AM/PM" with ampm in sFormat
      end if

      Replace "YYYY" with Format$(rTime.wYear, "0000") in sFormat
      Replace "YY" with Right$(Format$(rTime.wYear, "0000"), 2) in sFormat
      Replace "MM" with Format$(rTime.wMonth, "00") in sFormat
      Replace "M" with Format$(rTime.wMonth, "0") in sFormat
      Replace "DD" with Format$(rTime.wDay, "00") in sFormat
      Replace "D" with Format$(rTime.wDay) in sFormat
      Replace "HH" with Format$(rTime.wHour, "00") in sFormat
      Replace "H" with Format$(rTime.wHour) in sFormat
      Replace "NN" with Format$(rTime.wMinute, "00") in sFormat
      Replace "N" with Format$(rTime.wMinute) in sFormat
      Replace "SS" with Format$(rTime.wSecond, "00") in sFormat
      Replace "S" with Format$(rTime.wSecond) in sFormat
      Function = sFormat
      FindClose hFind
   end if

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  FileSize
'
'  Returns file size in bytes or -1 if there's an error
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function FileSize alias "FileSize" (ByVal sFileName as String) Export as Long

   Dim hFind as Long
   Dim rData as WIN32_FIND_DATA

   hFind = FindFirstFile(sFileName + $nul, rData)
   if hFind = %INVALID_HANDLE_VALUE then
      Function = -1
   else
      Function = rData.nFileSizeLow
      FindClose hFind
   end if

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  ExtractFileName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function ExtractFileName alias "ExtractFileName" _
      ( ByVal sFileName As String ) Export As String

   Function = Parse$(sFileName, Any "\/:", -1)

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  FileExists
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function FileExists Alias "FileExists" _
      ( ByVal sFile As String ) Export As Long

   If Dir$(sFile) = "" Then
      Function = %FALSE
   Else
      Function = %TRUE
   End If

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  EnsureDirectoryExists
'
'  Creates all subdirectories if they don't already exist.
'  You can specify a full file name and or just the path.
'  If just the path is specified, it must end in a backslash
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub EnsureDirectoryExists Alias "EnsureDirectoryExists" _
      ( ByVal sFileName As String ) Export

   Dim i As Long
   Dim sDrive As String

   If Instr(sFileName, "\") < 1 Then Exit Sub

   '- All errors are ignored
   On Error Resume Next
   sDrive = ""
   For i = 1 To ParseCount(sFileName, "\") - 1
      If sDrive = "" Then
         sDrive = Parse$(sFileName, "\", 1)
      Else
         sDrive = sDrive + "\" + Parse$(sFileName, "\", i)
      End If
      MkDir sDrive
   Next i

   ErrClear
   On Error GoTo 0

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  ExtractPath
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function ExtractPath Alias "ExtractPath" _
      ( ByVal sFullFile As String ) Export As String

   '- This routine strips the file
   '  path from the full name.
   '
   Dim i As Long
   Dim iStartAccum As Long
   Dim iLen As Long
   Dim sFile As String
   Dim sAccum As String

   sFile = Ltrim$(Rtrim$(sFullFile))
   iLen = Len(sFile)
   sAccum = ""

   iStartAccum = %False
   If iLen > 0 Then
      For i = iLen To 1 Step -1
         If (Mid$(sFile, i, 1) = "\") Or _
            (Mid$(sFile, i, 1) = ":") Then
            iStartAccum = %True
         End If
         If iStartAccum = %True Then
            sAccum = Mid$(sFile, i, 1) + sAccum
         End If
      Next i
   End If

   sAccum = trim$(sAccum)
   if right$(sAccum, 1) <> "\" then sAccum = sAccum + "\"

   Function = sAccum

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetModuleName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function GetModuleName Alias "GetModuleName" _
      ( ByVal sModule as String ) Export as String

   Dim zFileName As Asciiz * 400

   GetModuleFileName GetModuleHandle(ByCopy sModule + $NUL), zFileName, 400
   Function = Trim$(zFileName)

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetExeName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function GetExeName Alias "GetExeName" Export As String

   Dim zFileName As Asciiz * 400

   GetModuleFileName ByVal %NULL, zFileName, 400
   Function = Trim$(zFileName)

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetAppPath
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function GetAppPath Alias "GetAppPath" Export  As String

   Dim sPath as String

   sPath = trim$(ExtractPath(GetExeName()))
   if right$(sPath, 1) <> "\" then sPath = sPath + "\"
   Function = sPath

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetTempFile
'
'  Takes a seed value (up to 4 characters)
'  and an extension (e.g. ".ddc") and
'  returns a unique file name.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function GetTempFile Alias "GetTempFile" _
      ( ByVal sSeed as String, ByVal sExt as String ) Export as String

   Dim iCount As Long
   Dim sName As String
   Dim sPath As String
   Dim sTest As String
   Dim sFormat As String
   Dim zPath As Asciiz * 300
   Dim zSeed As Asciiz * 100

   '- Get Temp Path
   GetTempPath SizeOf(zPath), zPath

   sPath = Trim$(zPath)
   If Right$(sPath, 1) <> "\" Then
      sPath$ = sPath + "\"
   End If

   If Len(sSeed) > 7 Then sSeed = Left$(sSeed,7)
   sFormat = String$(8 - Len(sSeed), "0")
   sName = ""
   For iCount = 1 To 9999
      sTest = sPath + sSeed + Format$(iCount, sFormat) + sExt
      If FileExists(sTest) = %False Then
         sName = sTest
         Exit For
      End If
   Next iCount

   Function = sName

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetTemporaryDir
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function GetTemporaryDir Alias "GetTemporaryDir" Export As String

   Dim zPath As Asciiz * 500
   Dim sPath As String

   GetTempPath SizeOf(zPath), zPath
   sPath = Trim$(zPath)
   If Right$(sPath, 1) <> "\" Then sPath = sPath + "\"
   Function = sPath

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetWindowsDir
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function GetWindowsDir Alias "GetWindowsDir" Export As String

   Dim zPath As Asciiz * 500
   Dim sPath As String

   GetWindowsDirectory zPath, SizeOf(zPath)
   sPath = Trim$(zPath)
   If Right$(sPath, 1) <> "\" Then sPath = sPath + "\"
   Function = sPath

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetSystemDir
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function GetSystemDir Alias "GetSystemDir" Export As String

   Dim zPath As Asciiz * 500
   Dim sPath As String

   GetSystemDirectory zPath, SizeOf(zPath)
   sPath = Trim$(zPath)
   If Right$(sPath, 1) <> "\" Then sPath = sPath + "\"
   Function = sPath

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  DeleteFileSpec
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub DeleteFileSpec Alias "DeleteFileSpec" _
      ( ByVal sFileSpec As String ) Export

   Dim i As Long
   Dim iCount As Long
   Dim sF As String
   Dim sPath As String
   Dim sFiles() As String

   On Error Resume Next

   ReDim sFiles(0 To 0) As String
   iCount = 0

   sPath = Trim$(ExtractPath(sFileSpec))
   If Right$(sPath, 1) <> "\" Then
      sPath = sPath + "\"
   End If

   sF = Dir$(sFileSpec)
   Do Until sF = ""
      iCount = iCount + 1
      ReDim Preserve sFiles(0 To iCount) As String
      sFiles(iCount) = sPath + sF
      sF = Dir$
   Loop
   For i = 1 To iCount
      errclear
      Kill sFiles(i)
   Next i

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbWriteFile
'     Writes a block of binary data to the file specified. If the
'     file exists, this fails.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function pbWriteFile Alias "pbWriteFile" _
      ( ByVal sFileName as String, ByVal sFileData as String ) _
        export as Long

   Dim iFF as Long

   if dir$(sFileName) <> "" then
      g_pb_file_LastError = "Output file exists."
      function = %false
      exit function
   end if

   iFF = FreeFile
   Open sFileName for binary lock read write as #iFF
   if err then
      g_pb_file_LastError = "Unable to open output file '" + _
            sFileName + "' error=" + Format$(err)
      Function = %false
      exit function
   end if

   Put #iFF,, sFileData

   Close #iFF

   Function = %true

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbzWriteFile
'
'  Writes a block of binary data to the file specified. If the
'  file exists, this fails.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function pbzWriteFile Alias "pbzWriteFile" _
      ( zFileName as Asciiz, zFileData as Asciiz ) export as Long

   Function = pbWriteFile( (zFileName), (zFileData) )

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  ShellDocument
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub ShellDocument alias "ShellDocument" (ByVal sFile as String) export

  ShellExecute( 0, ByVal %null, sFile + $nul, ByVal %null, ByVal %null, %SW_NORMAL

end sub

#endif
