'
'  fonts.bas
'
'  Simplified font handling routines.
'
'  CreateAngleFont
'  CreateScreenFont
'  CreateDCFont
'
'============================================================================

#If Not %Def(%PB_FONTS_BAS)

%PB_FONTS_BAS = -1

'
'  Parameter Contants
'============================================================================
%FONT_NORMAL      = 0
%FONT_BOLD        = 1
%FONT_ITALIC      = 2
%FONT_UNDER       = 4
%FONT_STRIKE      = 8

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  CreateAngleFont
'
'  This is a generic font creation routine that allows
'  fonts to be created with any Name, Size, Style,
'  Size and Angle. The other font routines in this module
'  wrap around this routine.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function CreateAngleFont alias "CreateAngleFont" _
      ( ByVal dcOutput As Long, fontName As String, _
        ByVal fontStyle As Integer, ByVal fontSize As Double, _
        ByVal degrees As Integer ) Export As Long

   Dim zFontName As Asciiz * 100
   Dim iPPIY As Integer
   Dim iPPIX As Integer
   Dim rNewFont As LOGFONT

   iPPIX = GetDeviceCaps(dcOutput, %LOGPIXELSX)
   iPPIY = GetDeviceCaps(dcOutput, %LOGPIXELSY)

   zFontName = Trim$(fontName$)

   '- These FontSize values passed as POINTS. I must
   '  convert them into logical pixels.
   '
   rNewFont.lfHeight = -1 * (fontSize * iPPIY ) / 72
   rNewFont.lfWidth = 0
   rNewFont.lfEscapement = degrees * 10
   rNewFont.lfOrientation = degrees * 10

   If ( fontStyle And %FONT_BOLD ) = %FONT_BOLD Then
      rNewFont.lfWeight = %FW_BOLD
   Else
      rNewFont.lfWeight = %FW_NORMAL
   End If

   If (fontStyle And %FONT_ITALIC) = %FONT_ITALIC Then
     rNewFont.lfItalic = 1
   Else
     rNewFont.lfItalic = 0
   End If

   If (fontStyle And %FONT_STRIKE) = %FONT_STRIKE Then
     rNewFont.lfStrikeOut = 1
   Else
     rNewFont.lfStrikeOut = 0
   End If

   If (fontStyle And %FONT_UNDER) = %FONT_UNDER Then
     rNewFont.lfUnderline = 1
   Else
     rNewFont.lfUnderline = 0
   End If

   rNewFont.lfCharSet = %DEFAULT_CHARSET
   rNewFont.lfOutPrecision = %OUT_DEFAULT_PRECIS
   rNewFont.lfClipPrecision = %CLIP_DEFAULT_PRECIS
   rNewFont.lfQuality = %PROOF_QUALITY
   rNewFont.lfPitchAndFamily = %DEFAULT_PITCH + %FF_DONTCARE
   rNewFont.lfFaceName = Trim$(zFontName) + Chr$(0)

   '- First Retrieve Original Font
   zFontName = ""
   Function = CreateFontIndirect(rNewFont)

End Function


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  CreateScreenFont
'
'  The new font handle (WORD) is Returned
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~============================================================================
function CreateScreenFont alias "CreateScreenFont" _
      ( fontName As String, ByVal fontStyle As Integer, _
        ByVal fontSize As Double ) Export As Long
   '
   '  This function creates a font for use by the screen
   '  and returns it's handle. This object handle must be
   '  manually destroyed after use by calling DeleteObject.
   '
   Dim hDesktop As Long
   Dim dcDesktop As Long
   Dim iAngle As Integer

   '- Get the screen resolution
   hDesktop = GetDesktopWindow()
   dcDesktop = GetDC(hDesktop)
   iAngle = 0
   Function = CreateAngleFont(dcDesktop, fontName, fontStyle, fontSize, iAngle)
   ReleaseDC hDesktop, dcDesktop

End Function


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  CreateDCFont
'
'  This function creates a font for use by a device
'  and returns it's handle. This object handle must be
'  manually destroyed after use by calling DeleteObject.
'
'  The difference between this and CreateScreenFont, is that
'  this asks for the DC in order to retrieve results compatible
'  with any device. Create Screen Font retrieves a font good
'  enough for most window devices (list boxes, buttons, etc.)
'  while this routine retrieves fonts good enough for any device
'  (printers, etc.) with the disadvantage that you have to pass
'  it a valid DC.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function CreateDCFont alias "CreateDCFont" _
      ( ByVal dcOutput As Long, fontName As String, _
        ByVal fontStyle As Integer, ByVal fontSize As Double) Export As Long

   Dim iAngle As Integer

   iAngle = 0
   Function = CreateAngleFont (dcOutput, fontName, fontStyle, fontSize, iAngle)

End Function

#EndIf

